#ifndef _ETATS_SUPPORT_H
#define _ETATS_SUPPORT_H (1)

#include <glib.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */


/* START_DECLARATION */
gchar *		etats_support_get_titre 	(gint report_number);
/* END_DECLARATION */
#endif
