#ifndef _UTILS_DATES_CUNIT_H
#define _UTILS_DATES_CUNIT_H (1)

#include <CUnit/Basic.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */

/* START_DECLARATION */
CU_pSuite utils_dates_cunit_create_suite ( void );
/* END_DECLARATION */

#endif /*_UTILS_DATES_CUNIT_H */
