#ifndef _PRINT_TREE_VIEW_LIST_H
#define _PRINT_TREE_VIEW_LIST_H

#include <gtk/gtk.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */



/* START_DECLARATION */
gboolean	print_tree_view_list	(GtkWidget *menu_item,
									 GtkTreeView *tree_view);

/* END_DECLARATION */


#endif /*_PRINT_TREE_VIEW_LIST_H */

