#ifndef _TIP_H
#define _TIP_H (1)

#include <glib.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */


/* START_DECLARATION */
void	display_tip			(gboolean force);
void	force_display_tip	(void);
/* END_DECLARATION */
#endif
