#ifndef _GSB_ASSISTANT_ARCHIVE_EXPORT_H
#define _GSB_ASSISTANT_ARCHIVE_EXPORT_H (1)

#include <gtk/gtk.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */

/* START_DECLARATION */
GtkResponseType 	gsb_assistant_archive_export_run 	(void);
/* END_DECLARATION */
#endif
