#ifndef _TRAITEMENT_VARIABLES_H
#define _TRAITEMENT_VARIABLES_H (1)

#include <glib.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */


/*START_DECLARATION*/
void	free_variables		(void);
void	init_variables		(void);
/*END_DECLARATION*/


#endif

