#ifndef _GSB_DATA_ACCOUNT_CUNIT_H
#define _GSB_DATA_ACCOUNT_CUNIT_H (1)

#include <CUnit/Basic.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */

/* START_DECLARATION */
CU_pSuite gsb_data_account_cunit_create_suite(void);
/* END_DECLARATION */

#endif /*_GSB_DATA_ACCOUNT_CUNIT_H */
