#ifndef _MOUSE_H
#define _MOUSE_H (1)

/* Fichier de définition de constantes relatives à la souris mouse.h */

/* possible values for a GdkEventButton -> button
 * cf GdkEventButton structure in gnome library */
#define LEFT_BUTTON 1
#define MIDDLE_BUTTON 2
#define RIGHT_BUTTON 3


/* START_INCLUDE_H */
/* END_INCLUDE_H */


/*START_DECLARATION*/
/*END_DECLARATION*/


#endif

