#ifndef _UTILS_OPERATIONS_H
#define _UTILS_OPERATIONS_H (1)

#include <glib.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */


/* START_DECLARATION */
void	delete_transaction_in_trees		(gint transaction_number);
void	update_transaction_in_trees		(gint transaction_number);
/* END_DECLARATION */
#endif
