#ifndef ETATS_CSV_H
#define ETATS_CSV_H

#include "structures.h"

extern struct EtatAffichage csv_affichage;


/* START_INCLUDE_H */
#include "etats_affiche.h"
/* END_INCLUDE_H */


/* START_DECLARATION */
/* END_DECLARATION */

#endif
