#ifndef _ECHEANCIER_INFOS_H
#define _ECHEANCIER_INFOS_H (1)

#include <gtk/gtk.h>

/* START_INCLUDE_H */
#include "gsb_scheduler_list.h"
/* END_INCLUDE_H */


/* START_DECLARATION */
GtkWidget *	gsb_calendar_new 		(void);
gboolean 	gsb_calendar_update 	(void);
/* END_DECLARATION */
#endif
