#ifndef _FILE_OBFUSCATE_H
#define _FILE_OBFUSCATE_H (1)

#include <glib.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */

/* START_DECLARATION */
gboolean 	file_obfuscate_run	(void);
/* END_DECLARATION */
#endif
