#ifndef _PRINT_TRANSACTIONS_LIST_H
#define _PRINT_TRANSACTIONS_LIST_H (1)

#include <gtk/gtk.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */

/* START_DECLARATION */
gboolean	print_transactions_list_apply	(GtkPrintOperation *operation,
											 GtkWidget *widget,
											 gpointer null);
gboolean	print_transactions_list			(GtkWidget *button,
											 gpointer null);
/* END_DECLARATION */
#endif
