#ifndef _PRINT_DIALOG_CONFIG_H
#define _PRINT_DIALOG_CONFIG_H (1)

#include <glib-object.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */

/* START_DECLARATION */
void	print_dialog_config		(GCallback begin_callback,
								 GCallback draw_callback,
								 gchar *custom_tab_label,
								 GCallback custom_tab_callback,
								 GCallback custom_apply_callback,
								 gpointer data);
/* END_DECLARATION */
#endif
