#ifndef __PREFS_PAGE_PAYMENT_METHOD_H__
#define __PREFS_PAGE_PAYMENT_METHOD_H__

#include <gtk/gtk.h>
#include <glib.h>

/*START_INCLUDE*/
#include "grisbi_prefs.h"
/*END_INCLUDE*/

G_BEGIN_DECLS

#define PREFS_PAGE_PAYMENT_METHOD_TYPE    	(prefs_page_payment_method_get_type ())
#define PREFS_PAGE_PAYMENT_METHOD(obj)    	(G_TYPE_CHECK_INSTANCE_CAST ((obj), PREFS_PAGE_PAYMENT_METHOD_TYPE, PrefsPagePaymentMethod))
#define PREFS_IS_PAGE_PAYMENT_METHOD(obj) 	(G_TYPE_CHECK_INSTANCE_TYPE((obj), PREFS_PAGE_PAYMENT_METHOD_TYPE))

typedef struct _PrefsPagePaymentMethod			PrefsPagePaymentMethod;
typedef struct _PrefsPagePaymentMethodClass		PrefsPagePaymentMethodClass;


struct _PrefsPagePaymentMethod
{
    GtkBox parent;
};

struct _PrefsPagePaymentMethodClass
{
    GtkBoxClass parent_class;
};

/* START_DECLARATION */
GType						prefs_page_payment_method_get_type				(void) G_GNUC_CONST;

PrefsPagePaymentMethod * 	prefs_page_payment_method_new					(GrisbiPrefs *prefs);

/* END_DECLARATION */

G_END_DECLS

#endif  /* __PREFS_PAGE_PAYMENT_METHOD_H__ */
