#ifndef _GSB_ASSISTANT_RECONCILE_CONFIG_H
#define _GSB_ASSISTANT_RECONCILE_CONFIG_H (1)

#include <gtk/gtk.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */


/* START_DECLARATION */
GtkResponseType 	gsb_assistant_reconcile_config_run 	(GtkWidget *tree_view);
/* END_DECLARATION */
#endif
