#ifndef ETATS_CONFIG_H
#define ETATS_CONFIG_H

#include <gtk/gtk.h>

#define TEXT_NORMAL (0<<0)
#define TEXT_BOLD   (1<<0)
#define TEXT_ITALIC (1<<1)
#define TEXT_HUGE   (1<<2)
#define TEXT_LARGE  (1<<3)
#define TEXT_SMALL  (1<<4)


/* START_INCLUDE_H */
/* END_INCLUDE_H */

/* START_DECLARATION */
gint		etats_config_personnalisation_etat		(void);
/* END_DECLARATION */


#endif /* ETATS_CONFIG_H */
