#ifndef _ETATS_GTKTABLE_H
#define _ETATS_GTKTABLE_H (1)

#include <gtk/gtk.h>

/* START_INCLUDE_H */
/* END_INCLUDE_H */


/* START_DECLARATION */
void			etats_gtktable_free_table_etat		(void);
GtkWidget *		etats_gtktable_get_table_etat			(void);
/* END_DECLARATION */
#endif
